@@ -306,12 +306,16 @@ __ebmb_insert(struct eb_root *root, struct ebmb_node *new, unsigned int len)
 	new_rght = eb_dotag(&new->node.branches, EB_RGHT);
 	new_leaf = eb_dotag(&new->node.branches, EB_LEAF);
 
-	/* Note: we can compare more bits than
-	 * the current node's because as long as they are identical, we
-	 * know we descend along the correct side.
-	 */
 	new->node.bit = bit;
-	diff = cmp_bits(new->key, old->key, bit);
+
+	/* Note: we can compare more bits than the current node's because as
+	 * long as they are identical, we know we descend along the correct
+	 * side. However we don't want to start to compare past the end.
+	 */
+	diff = 0;
+	if (((unsigned)bit >> 3) < len)
+		diff = cmp_bits(new->key, old->key, bit);
+
 	if (diff == 0) {
 		new->node.bit = -1; /* mark as new dup tree, just in case */
 
@@ -376,7 +380,7 @@ static forceinline struct ebmb_node *__ebmb_lookup_longest(struct eb_root *root,
 		if ((eb_gettag(troot) == EB_LEAF)) {
 			node = container_of(eb_untag(troot, EB_LEAF),
 					    struct ebmb_node, node.branches);
-			if (check_bits(x - pos, node->key, pos, node->node.pfx))
+			if (check_bits((unsigned char *)x - pos, node->key, pos, node->node.pfx))
 				goto not_found;
 
 			return node;
@@ -390,7 +394,7 @@ static forceinline struct ebmb_node *__ebmb_lookup_longest(struct eb_root *root,
 			 * value, and we walk down left, or it's a different
 			 * one and we don't have our key.
 			 */
-			if (check_bits(x - pos, node->key, pos, node->node.pfx))
+			if (check_bits((unsigned char *)x - pos, node->key, pos, node->node.pfx))
 				goto not_found;
 
 			troot = node->node.branches.b[EB_LEFT];
@@ -410,7 +414,7 @@ static forceinline struct ebmb_node *__ebmb_lookup_longest(struct eb_root *root,
 			 */
 			while (1) {
 				x++; pos++;
-				if (node->key[pos-1] ^ *(unsigned char*)(x-1))
+				if (node->key[pos-1] ^ *((unsigned char*)x - 1))
 					goto not_found; /* more than one full byte is different */
 				node_bit += 8;
 				if (node_bit >= 0)
@@ -469,7 +473,7 @@ static forceinline struct ebmb_node *__ebmb_lookup_prefix(struct eb_root *root,
 					    struct ebmb_node, node.branches);
 			if (node->node.pfx != pfx)
 				return NULL;
-			if (check_bits(x - pos, node->key, pos, node->node.pfx))
+			if (check_bits((unsigned char *)x - pos, node->key, pos, node->node.pfx))
 				return NULL;
 			return node;
 		}
@@ -484,7 +488,7 @@ static forceinline struct ebmb_node *__ebmb_lookup_prefix(struct eb_root *root,
 			 */
 			if (node->node.pfx != pfx)
 				return NULL;
-			if (check_bits(x - pos, node->key, pos, node->node.pfx))
+			if (check_bits((unsigned char *)x - pos, node->key, pos, node->node.pfx))
 				return NULL;
 
 			troot = node->node.branches.b[EB_LEFT];
@@ -504,7 +508,7 @@ static forceinline struct ebmb_node *__ebmb_lookup_prefix(struct eb_root *root,
 			 */
 			while (1) {
 				x++; pos++;
-				if (node->key[pos-1] ^ *(unsigned char*)(x-1))
+				if (node->key[pos-1] ^ *((unsigned char*)x - 1))
 					return NULL; /* more than one full byte is different */
 				node_bit += 8;
 				if (node_bit >= 0)
